c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c***********************************************************************
c							VERSION 0.9
c							14-FEB-2003
c  NAME:	BLOCKDEC    
c
c  DESCRIPTION: Primitive that performs block decimation with optional
c                 pipe checking
c
c  SWITCHES:
c		/DEBUG	 - debug info level         
c		/CHKPIPE - check output pipe for clogged pipe before filad
c		/QUIET   - don't warn on clogged output pipes
c
c  CALLING SEQUENCE:
c
c	BLOCKDEC (1) <INPUT>   - input data       
c                (2) <OUTPUT>  - block decimated data
c                (3) <BLKSIZE> - block size in elements
c                (4) <KEEP>    - number of succesive blocks to output
c                (5) <SKIP>    - number of blocks to skip between outputs
c
c  REVISION	PROGRAMMER	DESCRIPTION
c  --------	----------	-------------------------------------
c  02/14/03	rdh		baseline
c  03/11/03	rdh		modified for non-integer frame size
c  03/11/03	rdh		added framesize output             
c***********************************************************************
	program BLOCKDEC    
	implicit none
	include 'primitive.inc'

c	TYPE		NAME		Description
c	---------	-------------	--------------------------------
	record /HEADER/ HIN		!HCB for input 
	record /HEADER/ HOUT		!HCB for A output

	byte               bbuf(1)	!Input buffer
	pointer(ptr_bbuf,  bbuf)

	integer*4    debug			!debug switch
	integer*4    nget,ngot
	integer*4    nblksz,nkeep,nskip
	integer*4    ikeep
	integer*4    l_cons_skip
	logical*4    log_quiet,log_chkpipe
	logical*4    log_exact
	integer*4    idw_keep,  idw_skip	!Widget controls
	integer*4    istat_keep,istat_skip
	real*8       dblksz, eblk, eblksum	!Block size 
	real*8       dtemp
	character*40 nm_rslt

	logical*4 CHECK_PIPE		!prevent output clogging

	call M$CHECKIN()

c... switches              
        debug = M$GET_SWITCH('DEBUG')
	log_chkpipe = M$GET_PSWITCH('CHKPIPE')
	log_exact   = M$GET_PSWITCH('EXACT')   
	log_quiet   = M$GET_PSWITCH('QUIET')  
	nm_rslt = '                                        '
  	if (M$GET_PSWITCH('FRMSZ')) 
     -	  ikeep = M$GET_USWITCH('FRMSZ',nm_rslt)

c... open and test input data file/pipe
	call M$INITIALIZE(HIN)
	  HIN.file_name    = M$APICK(1)
	  HIN.type_allow   = '1000,2000'	
	  HIN.format_allow = 'S#,-SP,C#,-CP'
        call M$INOK(HIN)

c... grab command line stuff
	if (log_exact) then
	  dblksz = M$DPICK(3)
	  dblksz = MIN(MAX(0.0d0,dblksz),2147483647.0d0)
	  nblksz = NINT(dblksz) 
	else
	  nblksz = M$LPICK(3)
	  nblksz = MAX(1,nblksz)
	  dblksz = nblksz
	endif
	nkeep  = M$LPICK(4)
	nkeep  = MAX(1,nkeep)
	nskip  = M$LPICK(5)
	nskip  = MAX(0,nskip)

c... allocate output data file/pipe
	call M$INITIALIZE (HOUT)        
	call M$PROPAGATE  (HIN,HOUT)
	  HOUT.file_name = M$APICK(2)
	  HOUT.switch    = 'OPTIONAL'
	call M$ALLOCATE (HOUT)

c... malloc buffer
	ngot = MAX(1,HIN.bpe)*nblksz+1024
	call M$MALLOC(ngot,ptr_bbuf)

c... set dataflow 
   	if (.not. log_exact) then
      	  l_cons_skip = nblksz*(nskip)  
    	else
      	  dtemp = dblksz*DBLE(nskip)
      	  l_cons_skip = NINT(dtemp)
    	endif
    	HIN.xfer_len = nblksz
	HIN.cons_len = nblksz

c... initialize variables
	eblk    = 0.0d0                   
	eblksum = 0.0d0
	ikeep   = 0
	nget    = nblksz         
	ngot    = nget

c... setup realtime control widgets
	idw_keep = M$LWINIT('KEEP',nkeep,1,1073741824,1)
	idw_skip = M$LWINIT('SKIP',nskip,0,1073741824,1)

c... output framesize result
	if (nm_rslt(1:1) .ne. ' ')
     -	  call M$LRSLT(nm_rslt,nblksz)

	if (debug.gt.0) then
	  write(*,*) 'nblksz, dblksz: ',nblksz,dblksz
	  write(*,*) 'eblk:           ',eblk            
	  write(*,*) 'nkeep:          ',nkeep
	  write(*,*) 'nskip:          ',nskip
	  write(*,*) 'nget:           ',nget      
	  write(*,*) 'l_cons_skip:    ',l_cons_skip
	endif
 
	call M$SYNC()

c***********************************************************************
c BEGIN MAIN PROCESSING LOOP
c***********************************************************************

	DO WHILE ((ngot.eq.nget).and.(.not.Mc.break))
c	  Check for keep/skip change
	  istat_keep = M$LWGET(idw_keep,nkeep)
	  istat_skip = M$LWGET(idw_skip,nskip)
	  if ((istat_keep .ne. 0).or.(istat_skip .ne. 0)) then
	    nkeep = MAX(1,nkeep)
	    nskip = MAX(0,nskip)
	    if (.not. log_exact) then
	      l_cons_skip = nblksz*(nskip)  
	    else
	      dtemp = dblksz*DBLE(nskip)
	      l_cons_skip = NINT(dtemp)
	    endif
	    ikeep = 0
	  endif

c	  Update keep/skip counter
	  ikeep = ikeep + 1

c	  Grab buffers of data and discard if necessary
	  if (ikeep .gt. nkeep) then
	    if (log_exact) then
	      eblk    = dblksz*DBLE(nskip) - DBLE(l_cons_skip)
	      eblksum = eblksum + eblk
	    endif
	    HIN.cons_len = l_cons_skip
	    if (ABS(eblksum) .ge. 1.0d0) then
	      if (eblksum .lt. 0.0d0) then	!Insert sample
	        eblksum = eblksum + 1.0d0
	        HIN.cons_len = HIN.cons_len - 1
	      else				!Delete sample
	        eblksum = eblksum - 1.0d0
	        HIN.cons_len = HIN.cons_len + 1
	      endif
	    endif
	    ngot = 0
	    HIN.xfer_len = 0
	    call M$GRABX(HIN, bbuf, ngot)
c	    if (debug.gt.1) then
c	      write(*,100) 'Flow: ',HIN.offset,HIN.cons_len,eblk,eblksum
c	    endif
	    HIN.xfer_len = nblksz
	    HIN.cons_len = nblksz
	    ikeep = 1
	  endif

c	  Grab buffer of data and keep
	  if (log_exact) then
	    eblk = dblksz - DBLE(nblksz)
	    eblksum = eblksum + eblk
	  endif
	  HIN.cons_len = HIN.xfer_len
	  if (ABS(eblksum) .ge. 1.0d0) then
	    if (eblksum .lt. 0.0d0) then	!Insert sample
	      eblksum = eblksum + 1.0d0
	      HIN.cons_len = HIN.cons_len - 1
	    else				!Delete sample
	      eblksum = eblksum - 1.0d0
	      HIN.cons_len = HIN.cons_len + 1
	    endif
	  endif
	  ngot = 0
	  call M$GRABX(HIN, bbuf, ngot)

c	  if (debug.gt.1) then
c	    write(*,100) 'Flow: ',HIN.offset,HIN.cons_len,eblk,eblksum
100	    format(1X,1A,F15.1,I15,F20.12,F20.12)
c	  endif

c	  Put in output pipe/file
	  if (HOUT.open) then
	    if (log_chkpipe) then
	      if (CHECK_PIPE(HOUT,ngot)) then
 	        call M$FILAD(HOUT, bbuf, ngot)
	      else
	        if (.not. log_quiet) 
     -	          call M$INFO('BLOCKDEC dropped buffer. Pipe clogged')
	      endif
	    else
 	      call M$FILAD(HOUT, bbuf, ngot)
	    endif
	  endif

        ENDDO

c***********************************************************************
c END OF MAIN DO LOOP
c***********************************************************************
c... close files
        call M$CLOSE(HIN)
        call M$CLOSE(HOUT)

        call M$RETURN()
        END


c***********************************************************************
c CHECK_PIPE Function
c***********************************************************************
        logical*4 function CHECK_PIPE (HCB,ielems)
        implicit none
        include 'midas.inc'
        include 'headers.inc'
        record /HEADER/ HCB
        integer*4       ielems
        real*8          dpsize,dread,dwrite,dbmax,dcons

        if (HCB.pipe .ne. 0) then
          call M$PINDICES(%val(HCB.lph.p), dpsize, dread, dwrite)
          dbmax = dpsize - DBLE(ielems)*HCB.dbpe
          dcons = dwrite - dread
          if (dcons .gt. dbmax) then
            CHECK_PIPE = .false.
          else
            CHECK_PIPE = .true.
          endif
        else
          CHECK_PIPE = .true.
        endif

        return
        end
