c        1         2         3         4         5         6         7
c23456789012345678901234567890123456789012345678901234567890123456789012
c***********************************************************************
c
c							VERSION 0.9
c							23-MAY-2001
c
c  NAME:	MAX2D     
c
c  CLASSIFIED:	NONE
c
c  DESCRIPTION: Primitive which does a "maximum hold" function
c                 on type 2000 input data    
c
c  SWITCHES:
c		/DEBUG	 - output debug information
c		/MODE    - determines output mode 
c                            1 = output updated maximum
c                            2 = output is mux of input with maximum
c                            3 = output is single frame with maximums
c		/ABSCMIN - minimum abscissa for output widgets/results
c		/ABSCMAX - maximum abscissa for output widgets/results
c		/RSLT=   - prefix for output result names
c  WIDGETS:
c             1:RESET   - max reset widget (1=reset, 0=normal)
c             2:MAXVAL  - maximum value updated over each frame 
c             3:MAXIDX  - subframe index of maximum value 
c             4:MAXABSC - subframe abscissa of maximum value
c
c  CALLING SEQUENCE:
c
c	MAX2D (1) <INPUT> 
c	      (2) <OUTPUT> 
c
c  REVISION	PROGRAMMER	DESCRIPTION
c  ----------	----------	-------------------------------------
c  2001-05-23	rdh		baseline taken from MAXPSD
c  2004-03-15	rdh		updated with new widgets
c
c***********************************************************************
	program MAX2D    
	implicit none
	include 'primitive.inc'

	integer*4  mode_maxout,    mode_muxout,   mode_oneshot
	parameter (mode_maxout=1,  mode_muxout=2, mode_oneshot=3)
	integer*4  mode_perframe
	parameter (mode_perframe=4 )

c	TYPE		NAME		Description
c	---------	-------------	--------------------------------
	record /HEADER/ HIN		!HCB for input 
	record /HEADER/ HOUT		!HCB for PSD output

	integer*4	sw_debug	!Input debug switch
	integer*4	sw_mode		!Input mode switch
	integer*4	sw_rslt		!Output results prefix
	integer*4	sw_abscmin	!Abscissa minimum switch
	integer*4	sw_abscmax	!Abscissa minimum switch

	integer*4	idw_reset	!MaxHold reset widget ID
	integer*4	idw_maxval	!Maximum value over a frame
	integer*4	idw_maxidx	!Index of maximum value 
	integer*4	idw_maxabsc	!Abscissa of maximum value

	real*4		bufA(1)
	real*4		bufB(1)
	real*4		bufC(1)
	pointer(ptr_bufA, bufA)
	pointer(ptr_bufB, bufB)
	pointer(ptr_bufC, bufC)

	logical*4	log_rsltchng
	integer*4	i
	integer*4	nget,ngot
	integer*4	istat
	integer*4       idxmax
	integer*4       imaxidx
	integer*4       iabscmin
	integer*4       iabscmax
	real*4          rmaxval
	complex*8       cmaxval
	real*8          dmaxval, dmaxval_prev
	real*8          dmaxidx, dmaxidx_prev
	real*8          dmaxabsc, dmaxabsc_prev
	real*8          dabscmin
	real*8          dabscmax
	real*8          dtmp    
	character*40	c_rslt

	call M$CHECKIN()

c... get switches
	sw_debug   = M$GET_SWITCH('DEBUG')
  	sw_mode    = M$GET_SWITCH('MODE')
  	sw_mode    = MIN(MAX(mode_maxout,sw_mode),mode_perframe)
	sw_rslt    = M$GET_USWITCH('RSLT',c_rslt)
	if (M$GET_PSWITCH('ABSCMIN')) then
	  sw_abscmin = 1                         
	  dabscmin   = M$GET_DSWITCH('ABSCMIN')
	else
	  sw_abscmin = -1                        
	  dabscmin   =  0.0d0
	endif
	if (M$GET_PSWITCH('ABSCMAX')) then
	  sw_abscmax = 1
	  dabscmax   = M$GET_DSWITCH('ABSCMAX')
	else
	  sw_abscmax = -1
	  dabscmax   =  0.0d0
	endif

        if (sw_debug.ge.0)
     -    call M$INFO('----==== MAX2D Debug Enabled ====----')

c... check input file/pipe
	call M$INITIALIZE(HIN)
	  HIN.file_name    = M$APICK(1)
	  HIN.type_allow   = '2000'	
	  HIN.format_allow = 'SF,CF'
        call M$INOK(HIN)

c... Setup and allocate output file/pipe
	call M$INITIALIZE (HOUT)        
	call M$PROPAGATE(HIN,HOUT)
	  HOUT.file_name = M$APICK(2)
	  HOUT.switch    = 'OPTIONAL'
	  if (sw_mode .eq. mode_muxout) then
	    HOUT.ydelta = 0.5d0*HIN.ydelta
	  endif
	call M$ALLOCATE (HOUT)

c... Dataflow stuff
	nget = 1
	ngot = nget
	HIN.xfer_len =  nget          
	HIN.cons_len = -1     

c... Malloc input/output buffers
	istat = MAX(1,HIN.subsize)*MAX(4,HIN.bpa)+1024
	call M$MALLOC(istat, ptr_bufA)
	call M$MALLOC(istat, ptr_bufB)

c... Initialize buffers
	iabscmin = 1
	if (sw_abscmin .ge. 0) then
	  dabscmin = ((dabscmin-HIN.xstart)/MAX(1.0d-30,HIN.xdelta))+1.0d0
	  dtmp = DBLE(HIN.subsize)
	  iabscmin = MIN(MAX(1.0d0,dabscmin),dtmp)
	endif
	iabscmax = HIN.subsize
	if (sw_abscmax .ge. 0) then
	  dabscmax = ((dabscmax-HIN.xstart)/MAX(1.0d-30,HIN.xdelta))+1.0d0
	  dtmp = DBLE(HIN.subsize)
	  iabscmax = MIN(MAX(1.0d0,dabscmax),dtmp)
	endif
	iabscmin = MIN(iabscmin,iabscmax)

	call M$VTYPE('F')
	i = HIN.subsize
	if (HIN.format(1:2) .eq. 'CF') i = 2*HIN.subsize
	call M$VFILL(bufA,0.0,i)
	call M$VFILL(bufB,0.0,i)
	rmaxval =  0.0
	cmaxval = (0.0,0.0)
        idxmax  = HIN.subsize
	dmaxval_prev  = 0.0d0  
	dmaxidx_prev  = 0.0d0  
	dmaxabsc_prev = 0.0d0   

	if (sw_debug.gt.0) then
	  write(*,*) 'dabsc:   ',dabscmin,dabscmax
	  write(*,*) 'iabsc:   ',iabscmin,iabscmax
	  write(*,*) 'sw_absc: ',sw_abscmin,sw_abscmax
	endif
	            
c... setup real-time control widgets
	idw_reset   = M$LWINIT('RESET',0,0,1,1)
	idw_maxval  = M$DWINIT('MAXVAL',0.0d0,-1.0d30,1.0d30,0.1d0)
	idw_maxidx  = M$DWINIT('MAXIDX',0.0d0,0.0d0,1.0d30,1.0d0)
	idw_maxabsc = M$DWINIT('MAXABSC',0.0d0,-1.0d30,1.0d30,0.1d0)

c... Sync the pipes
	call M$SYNC()

c***********************************************************************
c BEGIN MAIN PROCESSING LOOP
c***********************************************************************

	DO WHILE ((ngot.eq.nget).and.(.not.Mc.break))
c	  Get new buffer of data
	  ngot = 0
	  call M$GRABX(HIN,bufA,ngot)

c	  Check for maxhold reset
	  i = 0
	  istat = M$LWGET(idw_reset,i)
	  ptr_bufC = ptr_bufB
	  if (istat .ne. 0) then
	    ptr_bufC = ptr_bufA
	    call M$LWPUT(idw_reset,0)
	  endif

c	  Check for per-frame behavior
	  if (sw_mode .eq. mode_perframe) then
	    ptr_bufC = ptr_bufA
	  endif

c	  Do maximum
	  if (ngot .eq. nget) then
	    if (HIN.format(1:2) .eq. 'CF') then
	      call vect_maxfunc_cf(bufA,bufC,bufB,idxmax,
     -	                           cmaxval,imaxidx,iabscmin,iabscmax)
	    else
	      call vect_maxfunc_sf(bufA,bufC,bufB,idxmax,
     -	                           rmaxval,imaxidx,iabscmin,iabscmax)
	    endif
	  endif

c	  Update realtime widgets
	  dmaxval = rmaxval
	  if (HIN.format(1:2) .eq. 'CF') dmaxval = ABS(cmaxval)
	  dmaxidx  = DBLE(imaxidx)
	  dmaxabsc = HIN.xstart + DBLE(imaxidx-1)*HIN.xdelta
	  call M$DWPUT(idw_maxval, dmaxval)
	  call M$DWPUT(idw_maxidx, dmaxidx)
	  call M$DWPUT(idw_maxabsc,dmaxabsc)

c	  Update results
	  if (sw_rslt .gt. 0) then
	    log_rsltchng = .false.
	    if (dmaxval  .ne. dmaxval_prev)  log_rsltchng = .true.
	    if (dmaxidx  .ne. dmaxidx_prev)  log_rsltchng = .true.
	    if (dmaxabsc .ne. dmaxabsc_prev) log_rsltchng = .true.
	    if (log_rsltchng) then
	      call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXVAL',dmaxval)
	      call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXIDX',dmaxidx)
	      call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXABSC',dmaxabsc)
	      dmaxval_prev  = dmaxval
	      dmaxidx_prev  = dmaxidx
	      dmaxabsc_prev = dmaxabsc
	    endif
	  endif

c	  Do filad
	  if (sw_mode .ne. mode_oneshot) then
	    if (sw_mode .eq. mode_muxout) then 
	      call M$FILAD(HOUT,bufA,ngot)
	    endif
	    call M$FILAD(HOUT,bufB,ngot)
	  endif
	ENDDO


	if (sw_mode .eq. mode_oneshot) then
	  if (HIN.format(1:2) .eq. 'CF') then
	    call vect_maxfunc_cf(bufB,bufB,bufB,idxmax,
     -	                         cmaxval,imaxidx,iabscmin,iabscmax)
	  else
	    call vect_maxfunc_sf(bufB,bufB,bufB,idxmax,
     -	                         rmaxval,imaxidx,iabscmin,iabscmax)
	  endif
	  ngot = 1
	  call M$FILAD(HOUT,bufB,ngot)
	  dmaxval = rmaxval
	  if (HIN.format(1:2) .eq. 'CF') dmaxval = ABS(cmaxval)
	  dmaxidx  = DBLE(imaxidx)
	  dmaxabsc = HIN.xstart + DBLE(imaxidx-1)*HIN.xdelta
	  call M$DWPUT(idw_maxval, dmaxval)
	  call M$DWPUT(idw_maxidx, dmaxidx)
	  call M$DWPUT(idw_maxabsc,dmaxabsc)
	  if (sw_rslt .gt. 0) then
	    call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXVAL',dmaxval)
	    call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXIDX',dmaxidx)
	    call M$DRSLT(c_rslt(1:sw_rslt)//'_MAXABSC',dmaxabsc)
	  endif
	endif

c***********************************************************************
c END OF MAIN DO LOOP
c***********************************************************************
c... close files
        call M$CLOSE(HIN)
        call M$CLOSE(HOUT)

        call M$RETURN()
        end
