/*--------------------------------------------------------------------------*/
/*  libserial.h - includes for linux serial I/O library                     */
/*--------------------------------------------------------------------------*/

#ifndef _LIBSERIAL_H
#define _LIBSERIAL_H

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _NOCDEFS
  typedef int          int_4;
  typedef unsigned int int_u4;
#else
#include "cdefs.h"
#endif

#define MAX_SERIALIO_STRINGLEN 1024

typedef struct 
{
  int_4   carriage;                       /* Carriage return                 */
  int_4   debuglevel;                     /* Debug output level              */
  int_4   quiet;                          /* Supress output to screen        */
  int_4   fd;                             /* File descriptor for port I/O    */
  int_4   iodelay;                        /* Global read/write delay in uSec */
  int_4   readdelay;                      /* Read delay in uSec              */
  int_4   writedelay;                     /* Write delay in uSec             */
  int_4   pad;                            /* Force 8-byte alignment          */
  char    devname[MAX_SERIALIO_STRINGLEN]; /* Linux serial device name        */

} LIBSERIAL_Object;

/* Function Prototypes */
#ifdef __cplusplus
#define EXTERN_CDEFINE extern "C"
#else
#define EXTERN_CDEFINE extern
#endif

EXTERN_CDEFINE int_4 serialport_create (void **SP);
EXTERN_CDEFINE int_4 serialport_init (void *SP, const char *DeviceName);
EXTERN_CDEFINE int_4 serialport_read (void *SP, char *bufRead, int_4 bufLen);
EXTERN_CDEFINE int_4 serialport_write (void *SP, char *bufWrite);
EXTERN_CDEFINE int_4 serialport_destroy (void **SP);

EXTERN_CDEFINE int_4 serialport_set_carriage (void *SP, int_4 i4val);
EXTERN_CDEFINE int_4 serialport_get_debuglevel (void *SP);
EXTERN_CDEFINE int_4 serialport_set_debuglevel (void *SP, int_4 i4val);
EXTERN_CDEFINE int_4 serialport_get_iodelay (void *SP);
EXTERN_CDEFINE int_4 serialport_set_iodelay (void *SP, int_4 i4val);
EXTERN_CDEFINE int_4 serialport_get_readdelay (void *SP);
EXTERN_CDEFINE int_4 serialport_set_readdelay (void *SP, int_4 i4val);
EXTERN_CDEFINE int_4 serialport_get_writedelay (void *SP);
EXTERN_CDEFINE int_4 serialport_set_writedelay (void *SP, int_4 i4val);

#endif /* _LIBSERIAL_H */
